$MIN_SCAN_GAP = 1000;
$MAX_SCAN_GAP = 3000;

$cardinalDirection[0] = "0 10000 0";    // N
$cardinalDirection[1] = "6000 6000 0";  // NE
$cardinalDirection[2] = "10000 0 0";    // E
$cardinalDirection[3] = "6000 -6000 0"; // SE
$cardinalDirection[4] = "0 -10000 0";   // S
$cardinalDirection[5] = "-6000 -6000 0";// SW
$cardinalDirection[6] = "-6000 0 0";    // W
$cardinalDirection[7] = "-6000 6000 0"; // NW
$NUM_CARDINALS = 8;

$MAX_LEADER_RANGE = 100;
$MIN_LEADER_RANGE = 0;
$CATCHING_SPEED = 15;//0.6;
$JOIN_SWARM_SPEED = 15;//0.4;
$SWARM_SPEED = 15;//0.5;
$LOST_SPEED = 15;//0.2;
$STATIONARY = 15;//0.01;


datablock PlayerData(HaroldDB : PlayerShape)
{
   boundingBox = "5.5 5.5 6";
   className = "NPC";
   shapeFile = "~/data/shapes/bots/harold.dts";
   maxForwardSpeed = 15;
};



function HaroldDB::onCollision(%this, %obj, %col)
{
   if(%col.getClassName() $= "Player")
   {
      // %client becomes client whose player hit logo 
      %client = %col.client;

      // increment score and message with score to client
      %client.HP = %client.HP + 10;

      commandToClient(%client, 'SetHPCounter', %client.HP);
   }
}





function HaroldDB::DoScan(%thisDB,%thisBot)
{
  if(!isObject(%thisBot))
    return;
  cancel(%thisBot.nextScan);
  
  %leader = %thisBot.FindLeader(%thisBot);
  %thisBot.SetMoveSpeed($CATCHING_SPEED);
  if (isObject(%leader))
  {
    %thisBot.SetAimObject( %thisBot.leader );
    error("LEADER FOUND!:"@%leader@" from:"@%thisBot);
    if (!%leader.aiPlayer) // real player, not ai
    {
      if ( (%leader != %thisBot.leader ))
      {
        %thisBot.leader = %leader;
        %thisBot.SetAimObject( %thisBot.leader );
        %thisBot.SetMoveSpeed($CATCHING_SPEED);
        %thisBot.Avoid();
	%thisBot.SetAimLocation(%leader.getTransform());
        %thisBot.SetMoveDestination(%leader.getTransform(),0);
        error("GONNA CATCH LEADER!...from:"@%thisBot);
      }
      else if (isObject(%thisBot.leader))
      {
        %thisBot.Avoid();
	%thisBot.SetAimLocation(%leader.getTransform());
        %thisBot.SetMoveDestination(%leader.getTransform(),0);
        error("TRYING TO CATCH LEADER!...from:"@%thisBot);
      }
    }
    else
    {
      %thisBot.leader = %leader;
      %thisBot.SetAimObject( %thisBot.leader );
      %thisBot.SetMoveSpeed($SWARM_SPEED);
      %thisBot.CentreOnSwarm();
      error("nearest object must be an AI!...from:"@%thisBot);
    }
  else
  {
    %thisBot.SetMoveSpeed($LOST_SPEED);
    %loc = $cardinalDirection[getRandom($NUM_CARDINALS-1)];
    %thisBot.SetAimLocation( %loc);
    %thisBot.Avoid();
    error("leaderless..:^(...from:"@%thisBot);
  } 
  %thisBot.nextScan = %thisDB.schedule($MIN_SCAN_GAP+getRandom($MAX_SCAN_GAP), "doScan", %thisBot);
}

function InitHaroldSwarm()
{
   new SimSet (AISwarmGroup);
   SpawnAHaroldSwarm();
}

function SpawnAHaroldSwarm()
{
   if ( (%role=nameToID("MissionGroup/AIDropPointsHarold/Swarm")) >= 0 )
   {
      %count=%role.getCount();
      for ( %i = 0; %i < %count; %i++)
      {
         spawnHarold(%i,"Swarm");
      }
   }
}


function spawnHarold(%index,%role)
{
  %thisBot = new AIPlayer() {
    dataBlock = HaroldDB;
    aiPlayer = true;
  };
  MissionCleanup.add(%thisBot);
  AISwarmGroup.add(%thisBot);
  
  %thisBot.SetAimObject( 0 );
  %spawn=pickHaroldSpawn(%index,%role);
  %thisBot.index = %index;
  %thisBot.SetTransform(%spawn.getTransform());
  %thisBot.SetEnergyLevel(60);
  %thisBot.role = %role;
  %thisBot.SetName(%spawn.getName() SPC %bot.role);
  %thisBot.SetShapeName( %thisBot.getName());
  
  echo("Added [" SPC %thisBot SPC "] :" SPC %thisBot.role SPC "#" SPC  %thisBot.index );

  %thisBot.SetMoveSpeed($LOST_SPEED);
  %thisBot.SetAimLocation( $cardinalDirection[getRandom($NUM_CARDINALS-1)]);
  %thisBot.nextScan = %thisBot.getDataBlock().schedule(2000, "doScan", %thisBot);

  return %thisBot;
}

function pickHaroldSpawn(%index,%role)
{

   %groupName = "MissionGroup/AIDropPointsHarold/" @ %role;
   %group = nameToID(%groupName);

   if (%group != -1) 
   {
      %count = %group.getCount();
      if (%count != 0) 
      {
        // %index = getRandom(%count-1);
         %spawn = %group.getObject(%index);
         return %spawn;
      }
      else
            DebugPrint("No spawn points found in " @ %groupName, "pickHaroldSpawn");
   }
   else
       DebugPrint("Missing spawn points group " @ %groupName, "pickHaroldSpawn");

   return %spawn;
}